# -*- coding: utf-8 -*-
"""
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import os
import sys
import tempfile

import salt.modules.cmdmod as cmdmod

# Import Salt Libs
import salt.utils.files
import salt.utils.platform
import salt.utils.stringutils
from salt.exceptions import CommandExecutionError
from salt.ext.six.moves import builtins  # pylint: disable=import-error
from salt.log import LOG_LEVELS
from tests.support.helpers import TstSuiteLoggingHandler

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, Mock, MockTimedProc, mock_open, patch
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, skipIf

DEFAULT_SHELL = "foo/bar"
MOCK_SHELL_FILE = "# List of acceptable shells\n" "\n" "/bin/bash\n"


class CMDMODTestCase(TestCase, LoaderModuleMockMixin):
    """
    Unit tests for the salt.modules.cmdmod module
    """

    def setup_loader_modules(self):
        return {cmdmod: {}}

    @classmethod
    def setUpClass(cls):
        cls.mock_loglevels = {
            "info": "foo",
            "all": "bar",
            "critical": "bar",
            "trace": "bar",
            "garbage": "bar",
            "error": "bar",
            "debug": "bar",
            "warning": "bar",
            "quiet": "bar",
        }

    @classmethod
    def tearDownClass(cls):
        del cls.mock_loglevels

    def test_render_cmd_no_template(self):
        """
        Tests return when template=None
        """
        self.assertEqual(cmdmod._render_cmd("foo", "bar", None), ("foo", "bar"))

    def test_render_cmd_unavailable_engine(self):
        """
        Tests CommandExecutionError raised when template isn't in the
        template registry
        """
        self.assertRaises(
            CommandExecutionError, cmdmod._render_cmd, "boo", "bar", "baz"
        )

    def test_check_loglevel_bad_level(self):
        """
        Tests return of providing an invalid loglevel option
        """
        with patch.dict(LOG_LEVELS, self.mock_loglevels):
            self.assertEqual(cmdmod._check_loglevel(level="bad_loglevel"), "foo")

    def test_check_loglevel_bad_level_not_str(self):
        """
        Tests the return of providing an invalid loglevel option that is not a string
        """
        with patch.dict(LOG_LEVELS, self.mock_loglevels):
            self.assertEqual(cmdmod._check_loglevel(level=1000), "foo")

    def test_check_loglevel_quiet(self):
        """
        Tests the return of providing a loglevel of 'quiet'
        """
        with patch.dict(LOG_LEVELS, self.mock_loglevels):
            self.assertEqual(cmdmod._check_loglevel(level="quiet"), None)

    def test_parse_env_not_env(self):
        """
        Tests the return of an env that is not an env
        """
        self.assertEqual(cmdmod._parse_env(None), {})

    def test_parse_env_list(self):
        """
        Tests the return of an env that is a list
        """
        ret = {"foo": None, "bar": None}
        self.assertEqual(ret, cmdmod._parse_env(["foo", "bar"]))

    def test_parse_env_dict(self):
        """
        Test the return of an env that is not a dict
        """
        self.assertEqual(cmdmod._parse_env("test"), {})

    def test_run_shell_is_not_file(self):
        """
        Tests error raised when shell is not available after _is_valid_shell error msg
        and os.path.isfile returns False
        """
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)):
                with patch("os.path.isfile", MagicMock(return_value=False)):
                    self.assertRaises(CommandExecutionError, cmdmod._run, "foo", "bar")

    def test_run_shell_file_no_access(self):
        """
        Tests error raised when shell is not available after _is_valid_shell error msg,
        os.path.isfile returns True, but os.access returns False
        """
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)):
                with patch("os.path.isfile", MagicMock(return_value=True)):
                    with patch("os.access", MagicMock(return_value=False)):
                        self.assertRaises(
                            CommandExecutionError, cmdmod._run, "foo", "bar"
                        )

    def test_run_runas_with_windows(self):
        """
        Tests error raised when runas is passed on windows
        """
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
                with patch.dict(cmdmod.__grains__, {"os": "fake_os"}):
                    self.assertRaises(
                        CommandExecutionError, cmdmod._run, "foo", "bar", runas="baz"
                    )

    def test_run_user_not_available(self):
        """
        Tests return when runas user is not available
        """
        mock_true = MagicMock(return_value=True)
        with patch("salt.modules.cmdmod._is_valid_shell", mock_true), patch(
            "os.path.isfile", mock_true
        ), patch("os.access", mock_true):
            self.assertRaises(
                CommandExecutionError, cmdmod._run, "foo", "bar", runas="baz"
            )

    def test_run_zero_umask(self):
        """
        Tests error raised when umask is set to zero
        """
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)):
                with patch("os.path.isfile", MagicMock(return_value=True)):
                    with patch("os.access", MagicMock(return_value=True)):
                        self.assertRaises(
                            CommandExecutionError, cmdmod._run, "foo", "bar", umask=0
                        )

    def test_run_invalid_umask(self):
        """
        Tests error raised when an invalid umask is given
        """
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)):
                with patch("os.path.isfile", MagicMock(return_value=True)):
                    with patch("os.access", MagicMock(return_value=True)):
                        self.assertRaises(
                            CommandExecutionError,
                            cmdmod._run,
                            "foo",
                            "bar",
                            umask="baz",
                        )

    def test_run_invalid_cwd_not_abs_path(self):
        """
        Tests error raised when cwd is not an absolute path
        """
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)):
                with patch("os.path.isfile", MagicMock(return_value=True)):
                    with patch("os.access", MagicMock(return_value=True)):
                        self.assertRaises(
                            CommandExecutionError, cmdmod._run, "foo", "bar"
                        )

    def test_run_invalid_cwd_not_dir(self):
        """
        Tests error raised when cwd is not a dir
        """
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)):
                with patch("os.path.isfile", MagicMock(return_value=True)):
                    with patch("os.access", MagicMock(return_value=True)):
                        with patch("os.path.isabs", MagicMock(return_value=True)):
                            self.assertRaises(
                                CommandExecutionError, cmdmod._run, "foo", "bar"
                            )

    def test_run_no_vt_os_error(self):
        """
        Tests error raised when not useing vt and OSError is provided
        """
        expected_error = "expect error"
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)):
                with patch("os.path.isfile", MagicMock(return_value=True)):
                    with patch("os.access", MagicMock(return_value=True)):
                        with patch(
                            "salt.utils.timed_subprocess.TimedProc",
                            MagicMock(side_effect=OSError(expected_error)),
                        ):
                            with self.assertRaises(CommandExecutionError) as error:
                                cmdmod.run("foo", cwd="/")
                            assert error.exception.args[0].endswith(
                                expected_error
                            ), repr(error.exception.args[0])

    def test_run_no_vt_io_error(self):
        """
        Tests error raised when not useing vt and IOError is provided
        """
        expected_error = "expect error"
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)):
                with patch("os.path.isfile", MagicMock(return_value=True)):
                    with patch("os.access", MagicMock(return_value=True)):
                        with patch(
                            "salt.utils.timed_subprocess.TimedProc",
                            MagicMock(side_effect=IOError(expected_error)),
                        ):
                            with self.assertRaises(CommandExecutionError) as error:
                                cmdmod.run("foo", cwd="/")
                            assert error.exception.args[0].endswith(
                                expected_error
                            ), repr(error.exception.args[0])

    @skipIf(salt.utils.platform.is_windows(), "Do not run on Windows")
    @skipIf(True, "Test breaks unittests runs")
    def test_run(self):
        """
        Tests end result when a command is not found
        """
        with patch("salt.modules.cmdmod._is_valid_shell", MagicMock(return_value=True)):
            with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)):
                with patch("os.path.isfile", MagicMock(return_value=True)):
                    with patch("os.access", MagicMock(return_value=True)):
                        ret = cmdmod._run("foo", cwd=os.getcwd(), use_vt=True).get(
                            "stderr"
                        )
                        self.assertIn("foo", ret)

    def test_is_valid_shell_windows(self):
        """
        Tests return if running on windows
        """
        with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
            self.assertTrue(cmdmod._is_valid_shell("foo"))

    @skipIf(salt.utils.platform.is_windows(), "Do not run on Windows")
    def test_is_valid_shell_none(self):
        """
        Tests return of when os.path.exists(/etc/shells) isn't available
        """
        with patch("os.path.exists", MagicMock(return_value=False)):
            self.assertIsNone(cmdmod._is_valid_shell("foo"))

    def test_is_valid_shell_available(self):
        """
        Tests return when provided shell is available
        """
        with patch("os.path.exists", MagicMock(return_value=True)):
            with patch("salt.utils.files.fopen", mock_open(read_data=MOCK_SHELL_FILE)):
                self.assertTrue(cmdmod._is_valid_shell("/bin/bash"))

    @skipIf(salt.utils.platform.is_windows(), "Do not run on Windows")
    def test_is_valid_shell_unavailable(self):
        """
        Tests return when provided shell is not available
        """
        with patch("os.path.exists", MagicMock(return_value=True)):
            with patch("salt.utils.files.fopen", mock_open(read_data=MOCK_SHELL_FILE)):
                self.assertFalse(cmdmod._is_valid_shell("foo"))

    @skipIf(salt.utils.platform.is_windows(), "Do not run on Windows")
    def test_os_environment_remains_intact(self):
        """
        Make sure the OS environment is not tainted after running a command
        that specifies runas.
        """
        with patch("pwd.getpwnam") as getpwnam_mock:
            with patch("subprocess.Popen") as popen_mock:
                environment = os.environ.copy()

                popen_mock.return_value = Mock(
                    communicate=lambda *args, **kwags: [b"", None],
                    pid=lambda: 1,
                    retcode=0,
                )

                with patch.dict(
                    cmdmod.__grains__, {"os": "Darwin", "os_family": "Solaris"}
                ):
                    if sys.platform.startswith(("freebsd", "openbsd")):
                        shell = "/bin/sh"
                    else:
                        shell = "/bin/bash"

                    cmdmod._run(
                        "ls", cwd=tempfile.gettempdir(), runas="foobar", shell=shell
                    )

                    environment2 = os.environ.copy()

                    self.assertEqual(environment, environment2)

                    if not salt.utils.platform.is_darwin():
                        getpwnam_mock.assert_called_with("foobar")

    @skipIf(not salt.utils.platform.is_darwin(), "applicable to macOS only")
    def test_shell_properly_handled_on_macOS(self):
        """
        cmd.run should invoke a new bash login only
        when bash is the default shell for the selected user
        """

        class _CommandHandler(object):
            """
            Class for capturing cmd
            """

            def __init__(self):
                self.cmd = None

            def clear(self):
                self.cmd = None

        cmd_handler = _CommandHandler()

        def mock_proc(__cmd__, **kwargs):
            cmd_handler.cmd = " ".join(__cmd__)
            return MagicMock(return_value=MockTimedProc(stdout=None, stderr=None))

        with patch("pwd.getpwnam") as getpwnam_mock:
            with patch("salt.utils.timed_subprocess.TimedProc", mock_proc):

                # User default shell is '/usr/local/bin/bash'
                user_default_shell = "/usr/local/bin/bash"
                with patch.dict(
                    cmdmod.__salt__,
                    {
                        "user.info": MagicMock(
                            return_value={"shell": user_default_shell}
                        )
                    },
                ):

                    cmd_handler.clear()
                    cmdmod._run(
                        "ls", cwd=tempfile.gettempdir(), runas="foobar", use_vt=False
                    )

                    self.assertRegex(
                        cmd_handler.cmd,
                        "{} -l -c".format(user_default_shell),
                        "cmd invokes right bash session on macOS",
                    )

                # User default shell is '/bin/zsh'
                user_default_shell = "/bin/zsh"
                with patch.dict(
                    cmdmod.__salt__,
                    {
                        "user.info": MagicMock(
                            return_value={"shell": user_default_shell}
                        )
                    },
                ):

                    cmd_handler.clear()
                    cmdmod._run(
                        "ls", cwd=tempfile.gettempdir(), runas="foobar", use_vt=False
                    )

                    self.assertNotRegex(
                        cmd_handler.cmd,
                        "bash -l -c",
                        "cmd does not invoke user shell on macOS",
                    )

    def test_run_cwd_doesnt_exist_issue_7154(self):
        """
        cmd.run should fail and raise
        salt.exceptions.CommandExecutionError if the cwd dir does not
        exist
        """
        cmd = "echo OHAI"
        cwd = "/path/to/nowhere"
        try:
            cmdmod.run_all(cmd, cwd=cwd)
        except CommandExecutionError:
            pass
        else:
            raise RuntimeError

    @skipIf(salt.utils.platform.is_windows(), "Do not run on Windows")
    @skipIf(salt.utils.platform.is_darwin(), "Do not run on MacOS")
    def test_run_cwd_in_combination_with_runas(self):
        """
        cmd.run executes command in the cwd directory
        when the runas parameter is specified
        """
        cmd = "pwd"
        cwd = "/tmp"
        runas = os.getlogin()

        with patch.dict(cmdmod.__grains__, {"os": "Darwin", "os_family": "Solaris"}):
            stdout = cmdmod._run(cmd, cwd=cwd, runas=runas).get("stdout")
        self.assertEqual(stdout, cwd)

    def test_run_all_binary_replace(self):
        """
        Test for failed decoding of binary data, for instance when doing
        something silly like using dd to read from /dev/urandom and write to
        /dev/stdout.
        """
        # Since we're using unicode_literals, read the random bytes from a file
        rand_bytes_file = os.path.join(RUNTIME_VARS.BASE_FILES, "random_bytes")
        with salt.utils.files.fopen(rand_bytes_file, "rb") as fp_:
            stdout_bytes = fp_.read()

        # kitchen-salt uses unix2dos on all the files before copying them over
        # to the vm that will be running the tests. It skips binary files though
        # The file specified in `rand_bytes_file` is detected as binary so the
        # Unix-style line ending remains. This should account for that.
        stdout_bytes = stdout_bytes.rstrip() + os.linesep.encode()

        # stdout with the non-decodable bits replaced with the unicode
        # replacement character U+FFFD.
        stdout_unicode = "\ufffd\x1b\ufffd\ufffd" + os.linesep
        stderr_bytes = (
            os.linesep.encode().join(
                [
                    b"1+0 records in",
                    b"1+0 records out",
                    b"4 bytes copied, 9.1522e-05 s, 43.7 kB/s",
                ]
            )
            + os.linesep.encode()
        )
        stderr_unicode = stderr_bytes.decode()

        proc = MagicMock(
            return_value=MockTimedProc(stdout=stdout_bytes, stderr=stderr_bytes)
        )
        with patch("salt.utils.timed_subprocess.TimedProc", proc):
            ret = cmdmod.run_all(
                "dd if=/dev/urandom of=/dev/stdout bs=4 count=1", rstrip=False
            )

        self.assertEqual(ret["stdout"], stdout_unicode)
        self.assertEqual(ret["stderr"], stderr_unicode)

    def test_run_all_none(self):
        """
        Tests cases when proc.stdout or proc.stderr are None. These should be
        caught and replaced with empty strings.
        """
        proc = MagicMock(return_value=MockTimedProc(stdout=None, stderr=None))
        with patch("salt.utils.timed_subprocess.TimedProc", proc):
            ret = cmdmod.run_all("some command", rstrip=False)

        self.assertEqual(ret["stdout"], "")
        self.assertEqual(ret["stderr"], "")

    def test_run_all_unicode(self):
        """
        Ensure that unicode stdout and stderr are decoded properly
        """
        stdout_unicode = "Here is some unicode: спам"
        stderr_unicode = "Here is some unicode: яйца"
        stdout_bytes = stdout_unicode.encode("utf-8")
        stderr_bytes = stderr_unicode.encode("utf-8")

        proc = MagicMock(
            return_value=MockTimedProc(stdout=stdout_bytes, stderr=stderr_bytes)
        )

        with patch("salt.utils.timed_subprocess.TimedProc", proc), patch.object(
            builtins, "__salt_system_encoding__", "utf-8"
        ):
            ret = cmdmod.run_all("some command", rstrip=False)

        self.assertEqual(ret["stdout"], stdout_unicode)
        self.assertEqual(ret["stderr"], stderr_unicode)

    def test_run_all_output_encoding(self):
        """
        Test that specifying the output encoding works as expected
        """
        stdout = "Æ"
        stdout_latin1_enc = stdout.encode("latin1")

        proc = MagicMock(return_value=MockTimedProc(stdout=stdout_latin1_enc))

        with patch("salt.utils.timed_subprocess.TimedProc", proc), patch.object(
            builtins, "__salt_system_encoding__", "utf-8"
        ):
            ret = cmdmod.run_all("some command", output_encoding="latin1")

        self.assertEqual(ret["stdout"], stdout)

    def test_run_all_output_loglevel_quiet(self):
        """
        Test that specifying quiet for loglevel
        does not log the command.
        """
        stdout = b"test"
        proc = MagicMock(return_value=MockTimedProc(stdout=stdout))

        msg = "INFO:Executing command 'some command' in directory"
        with patch("salt.utils.timed_subprocess.TimedProc", proc):
            with TstSuiteLoggingHandler() as log_handler:
                ret = cmdmod.run_all("some command", output_loglevel="quiet")
                assert not [x for x in log_handler.messages if msg in x]

        self.assertEqual(ret["stdout"], salt.utils.stringutils.to_unicode(stdout))

    def test_run_all_output_loglevel_debug(self):
        """
        Test that specifying debug for loglevel
        does log the command.
        """
        stdout = b"test"
        proc = MagicMock(return_value=MockTimedProc(stdout=stdout))

        msg = "INFO:Executing command 'some command' in directory"
        with patch("salt.utils.timed_subprocess.TimedProc", proc):
            with TstSuiteLoggingHandler() as log_handler:
                ret = cmdmod.run_all("some command", output_loglevel="debug")
                assert [x for x in log_handler.messages if msg in x]

        self.assertEqual(ret["stdout"], salt.utils.stringutils.to_unicode(stdout))

    def test_run_chroot_mount(self):
        """
        Test cmdmod.run_chroot mount / umount balance
        """
        mock_mount = MagicMock()
        mock_umount = MagicMock()
        mock_run_all = MagicMock()
        with patch.dict(
            cmdmod.__salt__, {"mount.mount": mock_mount, "mount.umount": mock_umount}
        ):
            with patch("salt.modules.cmdmod.run_all", mock_run_all):
                cmdmod.run_chroot("/mnt", "cmd")
                self.assertEqual(mock_mount.call_count, 3)
                self.assertEqual(mock_umount.call_count, 3)

    def test_run_chroot_mount_bind(self):
        """
        Test cmdmod.run_chroot mount / umount balance with bind mount
        """
        mock_mount = MagicMock()
        mock_umount = MagicMock()
        mock_run_all = MagicMock()
        with patch.dict(
            cmdmod.__salt__, {"mount.mount": mock_mount, "mount.umount": mock_umount}
        ):
            with patch("salt.modules.cmdmod.run_all", mock_run_all):
                cmdmod.run_chroot("/mnt", "cmd", binds=["/var"])
                self.assertEqual(mock_mount.call_count, 4)
                self.assertEqual(mock_umount.call_count, 4)

    @skipIf(salt.utils.platform.is_windows(), "Skip test on Windows")
    def test_run_chroot_runas(self):
        """
        Test run_chroot when a runas parameter is provided
        """
        with patch.dict(
            cmdmod.__salt__, {"mount.mount": MagicMock(), "mount.umount": MagicMock()}
        ):
            with patch("salt.modules.cmdmod.run_all") as run_all_mock:
                cmdmod.run_chroot("/mnt", "ls", runas="foobar", shell="/bin/sh")
        run_all_mock.assert_called_with(
            "chroot --userspec foobar: /mnt /bin/sh -c ls",
            bg=False,
            clean_env=False,
            cwd=None,
            env=None,
            ignore_retcode=False,
            log_callback=None,
            output_encoding=None,
            output_loglevel="quiet",
            pillar=None,
            pillarenv=None,
            python_shell=True,
            reset_system_locale=True,
            rstrip=True,
            saltenv="base",
            shell="/bin/sh",
            stdin=None,
            success_retcodes=None,
            template=None,
            timeout=None,
            umask=None,
            use_vt=False,
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.npm as npm
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class NpmTestCase(TestCase, LoaderModuleMockMixin):

    """
    Test cases for salt.states.npm
    """

    def setup_loader_modules(self):
        return {npm: {"__opts__": {"test": False}}}

    # 'installed' function tests: 1

    def test_installed(self):
        """
        Test to verify that the given package is installed
        and is at the correct version.
        """
        name = "coffee-script"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock_err = MagicMock(side_effect=CommandExecutionError)
        mock_dict = MagicMock(return_value={name: {"version": "1.2"}})
        with patch.dict(npm.__salt__, {"npm.list": mock_err}):
            comt = "Error looking up 'coffee-script': "
            ret.update({"comment": comt})
            self.assertDictEqual(npm.installed(name), ret)

        with patch.dict(npm.__salt__, {"npm.list": mock_dict, "npm.install": mock_err}):
            with patch.dict(npm.__opts__, {"test": True}):
                comt = "Package(s) 'coffee-script' " "satisfied by coffee-script@1.2"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(npm.installed(name), ret)

            with patch.dict(npm.__opts__, {"test": False}):
                comt = "Package(s) 'coffee-script' " "satisfied by coffee-script@1.2"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(npm.installed(name), ret)

                comt = "Error installing 'n, p, m': "
                ret.update({"comment": comt, "result": False})
                self.assertDictEqual(npm.installed(name, "npm"), ret)

                with patch.dict(npm.__salt__, {"npm.install": mock_dict}):
                    comt = "Package(s) 'n, p, m' successfully installed"
                    ret.update(
                        {
                            "comment": comt,
                            "result": True,
                            "changes": {"new": ["n", "p", "m"], "old": []},
                        }
                    )
                    self.assertDictEqual(npm.installed(name, "npm"), ret)

    # 'removed' function tests: 1

    def test_removed(self):
        """
        Test to verify that the given package is not installed.
        """
        name = "coffee-script"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock_err = MagicMock(
            side_effect=[CommandExecutionError, {}, {name: ""}, {name: ""}]
        )
        mock_t = MagicMock(return_value=True)
        with patch.dict(npm.__salt__, {"npm.list": mock_err, "npm.uninstall": mock_t}):
            comt = "Error uninstalling 'coffee-script': "
            ret.update({"comment": comt})
            self.assertDictEqual(npm.removed(name), ret)

            comt = "Package 'coffee-script' is not installed"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(npm.removed(name), ret)

            with patch.dict(npm.__opts__, {"test": True}):
                comt = "Package 'coffee-script' is set to be removed"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(npm.removed(name), ret)

            with patch.dict(npm.__opts__, {"test": False}):
                comt = "Package 'coffee-script' was successfully removed"
                ret.update(
                    {"comment": comt, "result": True, "changes": {name: "Removed"}}
                )
                self.assertDictEqual(npm.removed(name), ret)

    # 'bootstrap' function tests: 1

    def test_bootstrap(self):
        """
        Test to bootstraps a node.js application.
        """
        name = "coffee-script"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock_err = MagicMock(side_effect=[CommandExecutionError, False, True])
        with patch.dict(npm.__salt__, {"npm.install": mock_err}):
            comt = "Error Bootstrapping 'coffee-script': "
            ret.update({"comment": comt})
            self.assertDictEqual(npm.bootstrap(name), ret)

            comt = "Directory is already bootstrapped"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(npm.bootstrap(name), ret)

            comt = "Directory was successfully bootstrapped"
            ret.update(
                {"comment": comt, "result": True, "changes": {name: "Bootstrapped"}}
            )
            self.assertDictEqual(npm.bootstrap(name), ret)

    # 'bootstrap' function tests: 1

    def test_cache_cleaned(self):
        """
        Test to verify that the npm cache is cleaned.
        """
        name = "coffee-script"

        pkg_ret = {"name": name, "result": False, "comment": "", "changes": {}}
        ret = {"name": None, "result": False, "comment": "", "changes": {}}

        mock_list = MagicMock(return_value=["~/.npm", "~/.npm/{0}/".format(name)])
        mock_cache_clean_success = MagicMock(return_value=True)
        mock_cache_clean_failure = MagicMock(return_value=False)
        mock_err = MagicMock(side_effect=CommandExecutionError)

        with patch.dict(npm.__salt__, {"npm.cache_list": mock_err}):
            comt = "Error looking up cached packages: "
            ret.update({"comment": comt})
            self.assertDictEqual(npm.cache_cleaned(), ret)

        with patch.dict(npm.__salt__, {"npm.cache_list": mock_err}):
            comt = "Error looking up cached {0}: ".format(name)
            pkg_ret.update({"comment": comt})
            self.assertDictEqual(npm.cache_cleaned(name), pkg_ret)

        mock_data = {"npm.cache_list": mock_list, "npm.cache_clean": MagicMock()}
        with patch.dict(npm.__salt__, mock_data):
            non_cached_pkg = "salt"
            comt = "Package {0} is not in the cache".format(non_cached_pkg)
            pkg_ret.update({"name": non_cached_pkg, "result": True, "comment": comt})
            self.assertDictEqual(npm.cache_cleaned(non_cached_pkg), pkg_ret)
            pkg_ret.update({"name": name})

            with patch.dict(npm.__opts__, {"test": True}):
                comt = "Cached packages set to be removed"
                ret.update({"result": None, "comment": comt})
                self.assertDictEqual(npm.cache_cleaned(), ret)

            with patch.dict(npm.__opts__, {"test": True}):
                comt = "Cached {0} set to be removed".format(name)
                pkg_ret.update({"result": None, "comment": comt})
                self.assertDictEqual(npm.cache_cleaned(name), pkg_ret)

            with patch.dict(npm.__opts__, {"test": False}):
                comt = "Cached packages successfully removed"
                ret.update(
                    {"result": True, "comment": comt, "changes": {"cache": "Removed"}}
                )
                self.assertDictEqual(npm.cache_cleaned(), ret)

            with patch.dict(npm.__opts__, {"test": False}):
                comt = "Cached {0} successfully removed".format(name)
                pkg_ret.update(
                    {"result": True, "comment": comt, "changes": {name: "Removed"}}
                )
                self.assertDictEqual(npm.cache_cleaned(name), pkg_ret)

        mock_data = {
            "npm.cache_list": mock_list,
            "npm.cache_clean": MagicMock(return_value=False),
        }
        with patch.dict(npm.__salt__, mock_data):
            with patch.dict(npm.__opts__, {"test": False}):
                comt = "Error cleaning cached packages"
                ret.update({"result": False, "comment": comt})
                ret["changes"] = {}
                self.assertDictEqual(npm.cache_cleaned(), ret)

            with patch.dict(npm.__opts__, {"test": False}):
                comt = "Error cleaning cached {0}".format(name)
                pkg_ret.update({"result": False, "comment": comt})
                pkg_ret["changes"] = {}
                self.assertDictEqual(npm.cache_cleaned(name), pkg_ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.unit.modules.archive_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os.path

# Import salt libs
import salt.modules.archive as archive
import salt.utils.path
from salt.exceptions import CommandNotFoundError

# Import 3rd-party libs
from salt.ext.six.moves import zip

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf


class ZipFileMock(MagicMock):
    def __init__(self, files=None, **kwargs):  # pylint: disable=W0231
        if files is None:
            files = ["salt"]
        MagicMock.__init__(self, **kwargs)
        self._files = files
        self.external_attr = 0o0777 << 16

    def namelist(self):
        return self._files


class ArchiveTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {archive: {"__grains__": {"id": 0}}}

    def test_tar(self):
        with patch("glob.glob", lambda pathname: [pathname]):
            with patch("salt.utils.path.which", lambda exe: exe):
                mock = MagicMock(return_value="salt")
                with patch.dict(archive.__salt__, {"cmd.run": mock}):
                    ret = archive.tar(
                        "-zcvf",
                        "foo.tar",
                        [
                            "/tmp/something-to-compress-1",
                            "/tmp/something-to-compress-2",
                        ],
                        template=None,
                    )
                    self.assertEqual(["salt"], ret)
                    mock.assert_called_once_with(
                        [
                            "tar",
                            "-zcvf",
                            "foo.tar",
                            "/tmp/something-to-compress-1",
                            "/tmp/something-to-compress-2",
                        ],
                        runas=None,
                        python_shell=False,
                        template=None,
                        cwd=None,
                    )

                mock = MagicMock(return_value="salt")
                with patch.dict(archive.__salt__, {"cmd.run": mock}):
                    ret = archive.tar(
                        "-zcvf",
                        "foo.tar",
                        "/tmp/something-to-compress-1,/tmp/something-to-compress-2",
                        template=None,
                    )
                    self.assertEqual(["salt"], ret)
                    mock.assert_called_once_with(
                        [
                            "tar",
                            "-zcvf",
                            "foo.tar",
                            "/tmp/something-to-compress-1",
                            "/tmp/something-to-compress-2",
                        ],
                        runas=None,
                        python_shell=False,
                        template=None,
                        cwd=None,
                    )

    def test_tar_raises_exception_if_not_found(self):
        with patch("salt.utils.path.which", lambda exe: None):
            mock = MagicMock(return_value="salt")
            with patch.dict(archive.__salt__, {"cmd.run": mock}):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.tar,
                    "zxvf",
                    "foo.tar",
                    "/tmp/something-to-compress",
                )
                self.assertFalse(mock.called)

    def test_gzip(self):
        mock = MagicMock(return_value="salt")
        with patch.dict(archive.__salt__, {"cmd.run": mock}):
            with patch("salt.utils.path.which", lambda exe: exe):
                ret = archive.gzip("/tmp/something-to-compress")
                self.assertEqual(["salt"], ret)
                mock.assert_called_once_with(
                    ["gzip", "/tmp/something-to-compress"],
                    runas=None,
                    python_shell=False,
                    template=None,
                )

    def test_gzip_raises_exception_if_not_found(self):
        mock = MagicMock(return_value="salt")
        with patch.dict(archive.__salt__, {"cmd.run": mock}):
            with patch("salt.utils.path.which", lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError, archive.gzip, "/tmp/something-to-compress"
                )
                self.assertFalse(mock.called)

    def test_gunzip(self):
        mock = MagicMock(return_value="salt")
        with patch.dict(archive.__salt__, {"cmd.run": mock}):
            with patch("salt.utils.path.which", lambda exe: exe):
                ret = archive.gunzip("/tmp/something-to-decompress.tar.gz")
                self.assertEqual(["salt"], ret)
                mock.assert_called_once_with(
                    ["gunzip", "/tmp/something-to-decompress.tar.gz"],
                    runas=None,
                    python_shell=False,
                    template=None,
                )

    def test_gunzip_raises_exception_if_not_found(self):
        mock = MagicMock(return_value="salt")
        with patch.dict(archive.__salt__, {"cmd.run": mock}):
            with patch("salt.utils.path.which", lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.gunzip,
                    "/tmp/something-to-decompress.tar.gz",
                )
                self.assertFalse(mock.called)

    def test_cmd_zip(self):
        with patch("glob.glob", lambda pathname: [pathname]):
            with patch("salt.utils.path.which", lambda exe: exe):
                mock = MagicMock(return_value="salt")
                with patch.dict(archive.__salt__, {"cmd.run": mock}):
                    ret = archive.cmd_zip(
                        "/tmp/salt.{{grains.id}}.zip",
                        "/tmp/tmpePe8yO,/tmp/tmpLeSw1A",
                        template="jinja",
                    )
                    self.assertEqual(["salt"], ret)
                    mock.assert_called_once_with(
                        [
                            "zip",
                            "-r",
                            "/tmp/salt.{{grains.id}}.zip",
                            "/tmp/tmpePe8yO",
                            "/tmp/tmpLeSw1A",
                        ],
                        runas=None,
                        python_shell=False,
                        template="jinja",
                        cwd=None,
                    )

                mock = MagicMock(return_value="salt")
                with patch.dict(archive.__salt__, {"cmd.run": mock}):
                    ret = archive.cmd_zip(
                        "/tmp/salt.{{grains.id}}.zip",
                        ["/tmp/tmpePe8yO", "/tmp/tmpLeSw1A"],
                        template="jinja",
                    )
                    self.assertEqual(["salt"], ret)
                    mock.assert_called_once_with(
                        [
                            "zip",
                            "-r",
                            "/tmp/salt.{{grains.id}}.zip",
                            "/tmp/tmpePe8yO",
                            "/tmp/tmpLeSw1A",
                        ],
                        runas=None,
                        python_shell=False,
                        template="jinja",
                        cwd=None,
                    )

    def test_zip(self):
        with patch("glob.glob", lambda pathname: [pathname]):
            with patch.multiple(
                os.path,
                **{
                    "isdir": MagicMock(return_value=False),
                    "exists": MagicMock(return_value=True),
                }
            ):
                with patch("zipfile.ZipFile", MagicMock()):
                    ret = archive.zip_(
                        "/tmp/salt.{{grains.id}}.zip",
                        "/tmp/tmpePe8yO,/tmp/tmpLeSw1A",
                        template="jinja",
                    )
                    expected = [
                        os.path.join("tmp", "tmpePe8yO"),
                        os.path.join("tmp", "tmpLeSw1A"),
                    ]
                    self.assertEqual(expected, ret)

    def test_zip_raises_exception_if_not_found(self):
        mock = MagicMock(return_value="salt")
        with patch.dict(archive.__salt__, {"cmd.run": mock}):
            with patch("salt.utils.path.which", lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.cmd_zip,
                    "/tmp/salt.{{grains.id}}.zip",
                    "/tmp/tmpePe8yO,/tmp/tmpLeSw1A",
                    template="jinja",
                )
                self.assertFalse(mock.called)

    def test_cmd_unzip(self):
        def _get_mock():
            """
            Create a new MagicMock for each scenario in this test, so that
            assert_called_once_with doesn't complain that the same mock object
            is called more than once.
            """
            return MagicMock(
                return_value={
                    "stdout": "salt",
                    "stderr": "",
                    "pid": 12345,
                    "retcode": 0,
                }
            )

        with patch("salt.utils.path.which", lambda exe: exe):

            mock = _get_mock()
            with patch.dict(archive.__salt__, {"cmd.run_all": mock}):
                ret = archive.cmd_unzip(
                    "/tmp/salt.{{grains.id}}.zip",
                    "/tmp/dest",
                    excludes="/tmp/tmpePe8yO,/tmp/tmpLeSw1A",
                    template="jinja",
                )
                self.assertEqual(["salt"], ret)
                mock.assert_called_once_with(
                    [
                        "unzip",
                        "/tmp/salt.{{grains.id}}.zip",
                        "-d",
                        "/tmp/dest",
                        "-x",
                        "/tmp/tmpePe8yO",
                        "/tmp/tmpLeSw1A",
                    ],
                    output_loglevel="debug",
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template="jinja",
                )

            mock = _get_mock()
            with patch.dict(archive.__salt__, {"cmd.run_all": mock}):
                ret = archive.cmd_unzip(
                    "/tmp/salt.{{grains.id}}.zip",
                    "/tmp/dest",
                    excludes=["/tmp/tmpePe8yO", "/tmp/tmpLeSw1A"],
                    template="jinja",
                )
                self.assertEqual(["salt"], ret)
                mock.assert_called_once_with(
                    [
                        "unzip",
                        "/tmp/salt.{{grains.id}}.zip",
                        "-d",
                        "/tmp/dest",
                        "-x",
                        "/tmp/tmpePe8yO",
                        "/tmp/tmpLeSw1A",
                    ],
                    output_loglevel="debug",
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template="jinja",
                )

            mock = _get_mock()
            with patch.dict(archive.__salt__, {"cmd.run_all": mock}):
                ret = archive.cmd_unzip(
                    "/tmp/salt.{{grains.id}}.zip",
                    "/tmp/dest",
                    excludes="/tmp/tmpePe8yO,/tmp/tmpLeSw1A",
                    template="jinja",
                    options="-fo",
                )
                self.assertEqual(["salt"], ret)
                mock.assert_called_once_with(
                    [
                        "unzip",
                        "-fo",
                        "/tmp/salt.{{grains.id}}.zip",
                        "-d",
                        "/tmp/dest",
                        "-x",
                        "/tmp/tmpePe8yO",
                        "/tmp/tmpLeSw1A",
                    ],
                    output_loglevel="debug",
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template="jinja",
                )

            mock = _get_mock()
            with patch.dict(archive.__salt__, {"cmd.run_all": mock}):
                ret = archive.cmd_unzip(
                    "/tmp/salt.{{grains.id}}.zip",
                    "/tmp/dest",
                    excludes=["/tmp/tmpePe8yO", "/tmp/tmpLeSw1A"],
                    template="jinja",
                    options="-fo",
                )
                self.assertEqual(["salt"], ret)
                mock.assert_called_once_with(
                    [
                        "unzip",
                        "-fo",
                        "/tmp/salt.{{grains.id}}.zip",
                        "-d",
                        "/tmp/dest",
                        "-x",
                        "/tmp/tmpePe8yO",
                        "/tmp/tmpLeSw1A",
                    ],
                    output_loglevel="debug",
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template="jinja",
                )

            mock = _get_mock()
            with patch.dict(archive.__salt__, {"cmd.run_all": mock}):
                ret = archive.cmd_unzip(
                    "/tmp/salt.{{grains.id}}.zip",
                    "/tmp/dest",
                    excludes=["/tmp/tmpePe8yO", "/tmp/tmpLeSw1A"],
                    template="jinja",
                    options="-fo",
                    password="asdf",
                )
                self.assertEqual(["salt"], ret)
                mock.assert_called_once_with(
                    [
                        "unzip",
                        "-P",
                        "asdf",
                        "-fo",
                        "/tmp/salt.{{grains.id}}.zip",
                        "-d",
                        "/tmp/dest",
                        "-x",
                        "/tmp/tmpePe8yO",
                        "/tmp/tmpLeSw1A",
                    ],
                    output_loglevel="quiet",
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template="jinja",
                )

    def test_unzip(self):
        mock = ZipFileMock()
        with patch("zipfile.ZipFile", mock):
            ret = archive.unzip(
                "/tmp/salt.{{grains.id}}.zip",
                "/tmp/dest",
                excludes="/tmp/tmpePe8yO,/tmp/tmpLeSw1A",
                template="jinja",
                extract_perms=False,
            )
            self.assertEqual(["salt"], ret)

    def test_unzip_raises_exception_if_not_found(self):
        mock = MagicMock(return_value="salt")
        with patch.dict(archive.__salt__, {"cmd.run": mock}):
            with patch("salt.utils.path.which", lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.cmd_unzip,
                    "/tmp/salt.{{grains.id}}.zip",
                    "/tmp/dest",
                    excludes="/tmp/tmpePe8yO,/tmp/tmpLeSw1A",
                    template="jinja",
                )
                self.assertFalse(mock.called)

    def test_rar(self):
        with patch("glob.glob", lambda pathname: [pathname]):
            with patch("salt.utils.path.which", lambda exe: exe):
                mock = MagicMock(return_value="salt")
                with patch.dict(archive.__salt__, {"cmd.run": mock}):
                    ret = archive.rar(
                        "/tmp/rarfile.rar", "/tmp/sourcefile1,/tmp/sourcefile2"
                    )
                    self.assertEqual(["salt"], ret)
                    mock.assert_called_once_with(
                        [
                            "rar",
                            "a",
                            "-idp",
                            "/tmp/rarfile.rar",
                            "/tmp/sourcefile1",
                            "/tmp/sourcefile2",
                        ],
                        runas=None,
                        python_shell=False,
                        template=None,
                        cwd=None,
                    )

                mock = MagicMock(return_value="salt")
                with patch.dict(archive.__salt__, {"cmd.run": mock}):
                    ret = archive.rar(
                        "/tmp/rarfile.rar", ["/tmp/sourcefile1", "/tmp/sourcefile2"]
                    )
                    self.assertEqual(["salt"], ret)
                    mock.assert_called_once_with(
                        [
                            "rar",
                            "a",
                            "-idp",
                            "/tmp/rarfile.rar",
                            "/tmp/sourcefile1",
                            "/tmp/sourcefile2",
                        ],
                        runas=None,
                        python_shell=False,
                        template=None,
                        cwd=None,
                    )

    def test_rar_raises_exception_if_not_found(self):
        mock = MagicMock(return_value="salt")
        with patch.dict(archive.__salt__, {"cmd.run": mock}):
            with patch("salt.utils.path.which", lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.rar,
                    "/tmp/rarfile.rar",
                    "/tmp/sourcefile1,/tmp/sourcefile2",
                )
                self.assertFalse(mock.called)

    @skipIf(salt.utils.path.which_bin(("unrar", "rar")) is None, "unrar not installed")
    def test_unrar(self):
        with patch(
            "salt.utils.path.which_bin",
            lambda exe: exe[0] if isinstance(exe, (list, tuple)) else exe,
        ):
            with patch(
                "salt.utils.path.which",
                lambda exe: exe[0] if isinstance(exe, (list, tuple)) else exe,
            ):
                mock = MagicMock(return_value="salt")
                with patch.dict(archive.__salt__, {"cmd.run": mock}):
                    ret = archive.unrar(
                        "/tmp/rarfile.rar", "/home/strongbad/", excludes="file_1,file_2"
                    )
                    self.assertEqual(["salt"], ret)
                    mock.assert_called_once_with(
                        [
                            "unrar",
                            "x",
                            "-idp",
                            "/tmp/rarfile.rar",
                            "-x",
                            "file_1",
                            "-x",
                            "file_2",
                            "/home/strongbad/",
                        ],
                        runas=None,
                        python_shell=False,
                        template=None,
                    )

                mock = MagicMock(return_value="salt")
                with patch.dict(archive.__salt__, {"cmd.run": mock}):
                    ret = archive.unrar(
                        "/tmp/rarfile.rar",
                        "/home/strongbad/",
                        excludes=["file_1", "file_2"],
                    )
                    self.assertEqual(["salt"], ret)
                    mock.assert_called_once_with(
                        [
                            "unrar",
                            "x",
                            "-idp",
                            "/tmp/rarfile.rar",
                            "-x",
                            "file_1",
                            "-x",
                            "file_2",
                            "/home/strongbad/",
                        ],
                        runas=None,
                        python_shell=False,
                        template=None,
                    )

    def test_unrar_raises_exception_if_not_found(self):
        with patch("salt.utils.path.which_bin", lambda exe: None):
            mock = MagicMock(return_value="salt")
            with patch.dict(archive.__salt__, {"cmd.run": mock}):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.unrar,
                    "/tmp/rarfile.rar",
                    "/home/strongbad/",
                )
                self.assertFalse(mock.called)

    def test_trim_files(self):
        with patch("salt.utils.path.which_bin", lambda exe: exe):
            source = "file.tar.gz"
            tmp_dir = "temp"
            files = [
                "\n".join(["file1"] * 200),
                "\n".join(["file2"] * 200),
                "this\nthat\nother",
                "this\nthat\nother",
                "this\nthat\nother",
            ]
            trim_opt = [True, False, 3, 1, 0]
            trim_100 = ["file1"] * 100
            trim_100.append("List trimmed after 100 files.")
            expected = [
                trim_100,
                ["file2"] * 200,
                ["this", "that", "other"],
                ["this", "List trimmed after 1 files."],
                ["List trimmed after 0 files."],
            ]

            for test_files, test_trim_opts, test_expected in zip(
                files, trim_opt, expected
            ):
                with patch.dict(
                    archive.__salt__, {"cmd.run": MagicMock(return_value=test_files)}
                ):
                    ret = archive.unrar(source, tmp_dir, trim_output=test_trim_opts)
                    self.assertEqual(ret, test_expected)



# coding: utf-8

# Python libs
from __future__ import absolute_import

import logging

# Salt libs
from salt.beacons import twilio_txt_msg

# Salt testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# Import 3rd Party libs
try:
    import twilio

    # Grab version, ensure elements are ints
    twilio_version = tuple([int(x) for x in twilio.__version_info__])
    if twilio_version > (5,):
        TWILIO_5 = False
    else:
        TWILIO_5 = True
    HAS_TWILIO = True
except ImportError:
    HAS_TWILIO = False


log = logging.getLogger(__name__)


class MockTwilioRestException(Exception):
    """
    Mock TwilioRestException class
    """

    def __init__(self):
        self.code = "error code"
        self.msg = "Exception error"
        self.status = "Not send"
        super(MockTwilioRestException, self).__init__(self.msg)


class MockMessages(object):
    """
    Mock SMS class
    """

    flag = None

    def __init__(self):
        self.sid = "011"
        self.price = "200"
        self.price_unit = "1"
        self.status = "Sent"
        self.num_segments = "2"
        self.num_media = "0"
        self.body = None
        self.date_sent = "01-01-2015"
        self.date_created = "01-01-2015"
        self.to = None
        self.from_ = None

    def create(self, body, to, from_):
        """
        Mock create method
        """
        msg = MockMessages()
        if self.flag == 1:
            raise MockTwilioRestException()
        msg.body = body
        msg.to = to
        msg.from_ = from_
        return msg

    def list(self, to):
        """
        Mock list method
        """
        msg = MockMessages()
        return [msg]

    def delete(self):
        """
        Mock delete method
        """
        return None


class MockSMS(object):
    """
    Mock SMS class
    """

    def __init__(self):
        self.messages = MockMessages()


class MockTwilioRestClient(object):
    """
    Mock TwilioRestClient class
    """

    def __init__(self):
        if TWILIO_5:
            self.sms = MockSMS()
        else:
            self.messages = MockMessages()


@skipIf(not HAS_TWILIO, "twilio.rest is not available")
class TwilioMsgTxtBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.twilio_txt_msg
    """

    def setup_loader_modules(self):
        return {twilio_txt_msg: {}}

    def test_validate_dictionary_config(self):
        """
        Test empty configuration
        """
        config = {}
        ret = twilio_txt_msg.validate(config)
        self.assertEqual(
            ret, (False, ("Configuration for twilio_txt_msg beacon must be a list."))
        )

    def test_validate_empty_config(self):
        """
        Test empty configuration
        """
        config = [{}]
        ret = twilio_txt_msg.validate(config)
        self.assertEqual(
            ret,
            (
                False,
                (
                    "Configuration for twilio_txt_msg "
                    "beacon must contain account_sid, "
                    "auth_token and twilio_number items."
                ),
            ),
        )

    def test_validate_missing_config_item(self):
        """
        Test empty configuration
        """
        config = [
            {
                "account_sid": "ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
                "twilio_number": "+15555555555",
            }
        ]

        ret = twilio_txt_msg.validate(config)
        self.assertEqual(
            ret,
            (
                False,
                (
                    "Configuration for twilio_txt_msg "
                    "beacon must contain account_sid, "
                    "auth_token and twilio_number items."
                ),
            ),
        )

    def test_receive_message(self):
        """
        Test receive a message
        """
        config = [
            {
                "account_sid": "ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
                "auth_token": "my_token",
                "twilio_number": "+15555555555",
            }
        ]

        ret = twilio_txt_msg.validate(config)
        self.assertEqual(ret, (True, "Valid beacon configuration"))

        _expected_return = [
            {
                "texts": [
                    {
                        "body": "None",
                        "images": [],
                        "from": "None",
                        "id": "011",
                        "sent": "01-01-2015",
                    }
                ]
            }
        ]
        mock = MagicMock(return_value=MockTwilioRestClient())
        with patch.object(twilio_txt_msg, "TwilioRestClient", mock):
            ret = twilio_txt_msg.beacon(config)
        self.assertEqual(ret, _expected_return)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.integration.shell.master
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

from __future__ import absolute_import

import pytest
import tests.integration.utils
from tests.integration.utils import testprogram
from tests.support.case import ShellCase
from tests.support.mixins import ShellCaseCommonTestsMixin
from tests.support.unit import skipIf


@skipIf(True, "This test file should be in an isolated test space.")
@pytest.mark.windows_whitelisted
class MasterTest(ShellCase, testprogram.TestProgramCase, ShellCaseCommonTestsMixin):

    _call_binary_ = "salt-master"

    def test_exit_status_unknown_user(self):
        """
        Ensure correct exit status when the master is configured to run as an unknown user.
        """

        master = testprogram.TestDaemonSaltMaster(
            name="unknown_user",
            configs={"master": {"map": {"user": "some_unknown_user_xyz"}}},
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        master.setup()
        stdout, stderr, status = master.run(
            args=["-d"], catch_stderr=True, with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status,
                "EX_NOUSER",
                message="unknown user not on system",
                stdout=stdout,
                stderr=tests.integration.utils.decode_byte_list(stderr),
            )
        finally:
            # Although the start-up should fail, call shutdown() to set the
            # internal _shutdown flag and avoid the registered atexit calls to
            # cause timeout exceptions and respective traceback
            master.shutdown()

    def test_exit_status_unknown_argument(self):
        """
        Ensure correct exit status when an unknown argument is passed to salt-master.
        """

        master = testprogram.TestDaemonSaltMaster(
            name="unknown_argument", parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        master.setup()
        stdout, stderr, status = master.run(
            args=["-d", "--unknown-argument"], catch_stderr=True, with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status,
                "EX_USAGE",
                message="unknown argument",
                stdout=stdout,
                stderr=tests.integration.utils.decode_byte_list(stderr),
            )
        finally:
            # Although the start-up should fail, call shutdown() to set the
            # internal _shutdown flag and avoid the registered atexit calls to
            # cause timeout exceptions and respective traceback
            master.shutdown()

    def test_exit_status_correct_usage(self):
        """
        Ensure correct exit status when salt-master starts correctly.
        """

        master = testprogram.TestDaemonSaltMaster(
            name="correct_usage", parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        master.setup()
        stdout, stderr, status = master.run(
            args=["-d"], catch_stderr=True, with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status,
                "EX_OK",
                message="correct usage",
                stdout=stdout,
                stderr=tests.integration.utils.decode_byte_list(stderr),
            )
        finally:
            master.shutdown(wait_for_orphans=3)

        # Do the test again to check does master shut down correctly
        # **Due to some underlying subprocessing issues with Minion._thread_return, this
        # part of the test has been commented out. Once these underlying issues have
        # been addressed, this part of the test should be uncommented. Work for this
        # issue is being tracked in https://github.com/saltstack/salt-jenkins/issues/378
        # stdout, stderr, status = master.run(
        #     args=['-d'],
        #     catch_stderr=True,
        #     with_retcode=True,
        # )
        # try:
        #     self.assert_exit_status(
        #         status, 'EX_OK',
        #         message='correct usage',
        #         stdout=stdout,
        #         stderr=tests.integration.utils.decode_byte_list(stderr)
        #     )
        # finally:
        #     master.shutdown(wait_for_orphans=3)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.quota as quota

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class QuotaTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.quota
    """

    def setup_loader_modules(self):
        return {quota: {}}

    # 'mode' function tests: 1

    def test_mode(self):
        """
        Test to set the quota for the system.
        """
        name = "/"
        mode = True
        quotatype = "user"

        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        mock_bool = MagicMock(side_effect=[True, False])
        mock = MagicMock(return_value={name: {quotatype: "on"}})
        with patch.dict(quota.__salt__, {"quota.get_mode": mock}):
            comt = "Quota for / already set to on"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(quota.mode(name, mode, quotatype), ret)

        mock = MagicMock(return_value={name: {quotatype: "off"}})
        with patch.dict(
            quota.__salt__, {"quota.get_mode": mock, "quota.on": mock_bool}
        ):
            with patch.dict(quota.__opts__, {"test": True}):
                comt = "Quota for / needs to be set to on"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(quota.mode(name, mode, quotatype), ret)

            with patch.dict(quota.__opts__, {"test": False}):
                comt = "Set quota for / to on"
                ret.update(
                    {"comment": comt, "result": True, "changes": {"quota": name}}
                )
                self.assertDictEqual(quota.mode(name, mode, quotatype), ret)

                comt = "Failed to set quota for / to on"
                ret.update({"comment": comt, "result": False, "changes": {}})
                self.assertDictEqual(quota.mode(name, mode, quotatype), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.artifactory as artifactory

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ArtifactoryTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.artifactory
    """

    def setup_loader_modules(self):
        return {artifactory: {}}

    # 'downloaded' function tests: 1

    def test_downloaded(self):
        """
        Test to ensures that the artifact from artifactory exists at
        given location.
        """
        name = "jboss"
        arti_url = "http://artifactory.intranet.example.com/artifactory"
        artifact = {
            "artifactory_url": arti_url,
            "artifact_id": "module",
            "repository": "libs-release-local",
            "packaging": "jar",
            "group_id": "com.company.module",
            "classifier": "sources",
            "version": "1.0",
        }

        ret = {"name": name, "result": False, "changes": {}, "comment": ""}

        mck = MagicMock(return_value={"status": False, "changes": {}, "comment": ""})
        with patch.dict(artifactory.__salt__, {"artifactory.get_release": mck}):
            self.assertDictEqual(artifactory.downloaded(name, artifact), ret)

        with patch.object(
            artifactory,
            "__fetch_from_artifactory",
            MagicMock(side_effect=Exception("error")),
        ):
            ret = artifactory.downloaded(name, artifact)
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["comment"], "error")

    # 'downloaded test=True' function tests: 1

    def test_downloaded_test_true(self):
        """
        Test to ensures that the artifact from artifactory exists at
        given location.
        """
        name = "jboss"
        arti_url = "http://artifactory.intranet.example.com/artifactory"
        artifact = {
            "artifactory_url": arti_url,
            "artifact_id": "module",
            "repository": "libs-release-local",
            "packaging": "jar",
            "group_id": "com.company.module",
            "classifier": "sources",
            "version": "1.0",
        }

        ret = {
            "name": name,
            "result": True,
            "changes": {},
            "comment": "Artifact would be downloaded from URL: http://artifactory.intranet.example.com/artifactory",
        }

        mck = MagicMock(return_value={"status": False, "changes": {}, "comment": ""})
        with patch.dict(artifactory.__salt__, {"artifactory.get_release": mck}):
            with patch.dict(artifactory.__opts__, {"test": True}):
                self.assertDictEqual(artifactory.downloaded(name, artifact), ret)

        with patch.object(
            artifactory,
            "__fetch_from_artifactory",
            MagicMock(side_effect=Exception("error")),
        ):
            ret = artifactory.downloaded(name, artifact)
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["comment"], "error")



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt libs
import salt.beacons.haproxy as haproxy
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Salt testing libs
from tests.support.unit import TestCase


class HAProxyBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.haproxy
    """

    def setup_loader_modules(self):
        return {haproxy: {"__context__": {}, "__salt__": {}}}

    def test_non_list_config(self):
        config = {}

        ret = haproxy.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for haproxy beacon must be a list.")
        )

    def test_empty_config(self):
        config = [{}]

        ret = haproxy.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for haproxy beacon requires backends.")
        )

    def test_no_servers(self):
        config = [{"backends": {"www-backend": {"threshold": 45}}}]
        ret = haproxy.validate(config)

        self.assertEqual(ret, (False, "Backends for haproxy beacon require servers."))

    def test_threshold_reached(self):
        config = [{"backends": {"www-backend": {"threshold": 45, "servers": ["web1"]}}}]
        ret = haproxy.validate(config)

        self.assertEqual(ret, (True, "Valid beacon configuration"))

        mock = MagicMock(return_value=46)
        with patch.dict(haproxy.__salt__, {"haproxy.get_sessions": mock}):
            ret = haproxy.beacon(config)
            self.assertEqual(ret, [{"threshold": 45, "scur": 46, "server": "web1"}])

    def test_threshold_not_reached(self):
        config = [
            {"backends": {"www-backend": {"threshold": 100, "servers": ["web1"]}}}
        ]
        ret = haproxy.validate(config)

        self.assertEqual(ret, (True, "Valid beacon configuration"))

        mock = MagicMock(return_value=50)
        with patch.dict(haproxy.__salt__, {"haproxy.get_sessions": mock}):
            ret = haproxy.beacon(config)
            self.assertEqual(ret, [])



# -*- coding: utf-8 -*-
"""
    tests.unit.version_test
    ~~~~~~~~~~~~~~~~~~~~~~~

    These tests are copied from python's source `Lib/distutils/tests/test_version.py`
    Some new examples were added and some adjustments were made to run tests in python 2 and 3
"""
# pylint: disable=string-substitution-usage-error

from __future__ import absolute_import, print_function, unicode_literals

import datetime
import os
import sys
import warnings

import salt.modules.cmdmod
import salt.utils.platform
import salt.utils.versions
import salt.version
from salt.ext import six
from salt.utils.versions import LooseVersion, StrictVersion
from tests.support.helpers import slowTest
from tests.support.mock import patch
from tests.support.paths import CODE_DIR
from tests.support.unit import TestCase, skipIf

if six.PY2:
    cmp_method = "__cmp__"
else:
    cmp_method = "_cmp"


class VersionTestCase(TestCase):
    def test_prerelease(self):
        version = StrictVersion("1.2.3a1")
        self.assertEqual(version.version, (1, 2, 3))
        self.assertEqual(version.prerelease, ("a", 1))
        self.assertEqual(six.text_type(version), "1.2.3a1")

        version = StrictVersion("1.2.0")
        self.assertEqual(six.text_type(version), "1.2")

    def test_cmp_strict(self):
        versions = (
            ("1.5.1", "1.5.2b2", -1),
            ("161", "3.10a", ValueError),
            ("8.02", "8.02", 0),
            ("3.4j", "1996.07.12", ValueError),
            ("3.2.pl0", "3.1.1.6", ValueError),
            ("2g6", "11g", ValueError),
            ("0.9", "2.2", -1),
            ("1.2.1", "1.2", 1),
            ("1.1", "1.2.2", -1),
            ("1.2", "1.1", 1),
            ("1.2.1", "1.2.2", -1),
            ("1.2.2", "1.2", 1),
            ("1.2", "1.2.2", -1),
            ("0.4.0", "0.4", 0),
            ("1.13++", "5.5.kw", ValueError),
            # Added by us
            ("1.1.1a1", "1.1.1", -1),
        )

        for v1, v2, wanted in versions:
            try:
                res = getattr(StrictVersion(v1), cmp_method)(StrictVersion(v2))
            except ValueError:
                if wanted is ValueError:
                    continue
                else:
                    raise AssertionError(
                        ("cmp(%s, %s) " "shouldn't raise ValueError") % (v1, v2)
                    )
            self.assertEqual(
                res, wanted, "cmp(%s, %s) should be %s, got %s" % (v1, v2, wanted, res)
            )

    def test_cmp(self):
        versions = (
            ("1.5.1", "1.5.2b2", -1),
            ("161", "3.10a", 1),
            ("8.02", "8.02", 0),
            ("3.4j", "1996.07.12", -1),
            ("3.2.pl0", "3.1.1.6", 1),
            ("2g6", "11g", -1),
            ("0.960923", "2.2beta29", -1),
            ("1.13++", "5.5.kw", -1),
            # Added by us
            ("3.10.0-514.el7", "3.10.0-514.6.1.el7", 1),
            ("2.2.2", "2.12.1", -1),
        )

        for v1, v2, wanted in versions:
            res = getattr(LooseVersion(v1), cmp_method)(LooseVersion(v2))
            self.assertEqual(
                res, wanted, "cmp(%s, %s) should be %s, got %s" % (v1, v2, wanted, res)
            )

    @skipIf(not salt.utils.platform.is_linux(), "only need to run on linux")
    @slowTest
    def test_spelling_version_name(self):
        """
        check the spelling of the version name for the release
        names in the salt.utils.versions.warn_until call
        """
        query = "salt.utils.versions.warn_until("
        names = salt.version.SaltStackVersion.NAMES

        cmd = "grep -lr {} -A 1 {}".format(query, os.path.join(CODE_DIR, "salt"))

        grep_call = salt.modules.cmdmod.run_stdout(cmd=cmd).split(os.linesep)

        for line in grep_call:
            num_cmd = salt.modules.cmdmod.run_stdout(
                "grep -c {0} {1}".format(query, line)
            )
            ver_cmd = salt.modules.cmdmod.run_stdout(
                "grep {0} {1} -A 1".format(query, line)
            )
            if "pyc" in line:
                break

            match = 0
            for key in names:
                if key in ver_cmd:
                    match = match + (ver_cmd.count(key))
            if "utils/__init__.py" in line:
                # work around for utils/__init__.py because
                # it includes the warn_utils function
                match = match + 1
            self.assertEqual(
                match,
                int(num_cmd),
                msg="The file: {0} has an "
                "incorrect spelling for the release name in the warn_utils "
                "call: {1}. Expecting one of these release names: "
                "{2}".format(line, ver_cmd, names),
            )


class VersionFuncsTestCase(TestCase):
    def test_compare(self):
        ret = salt.utils.versions.compare("1.0", "==", "1.0")
        self.assertTrue(ret)

        ret = salt.utils.versions.compare("1.0", "!=", "1.0")
        self.assertFalse(ret)

        with patch.object(salt.utils.versions, "log") as log_mock:
            ret = salt.utils.versions.compare(
                "1.0", "HAH I AM NOT A COMP OPERATOR! I AM YOUR FATHER!", "1.0"
            )
            self.assertTrue(log_mock.error.called)

    def test_kwargs_warn_until(self):
        # Test invalid version arg
        self.assertRaises(RuntimeError, salt.utils.versions.kwargs_warn_until, {}, [])

    def test_warn_until_warning_raised(self):
        # We *always* want *all* warnings thrown on this module
        warnings.filterwarnings("always", "", DeprecationWarning, __name__)

        def raise_warning(_version_info_=(0, 16, 0)):
            salt.utils.versions.warn_until(
                (0, 17), "Deprecation Message!", _version_info_=_version_info_
            )

        def raise_named_version_warning(_version_info_=(0, 16, 0)):
            salt.utils.versions.warn_until(
                "Hydrogen", "Deprecation Message!", _version_info_=_version_info_
            )

        # raise_warning should show warning until version info is >= (0, 17)
        with warnings.catch_warnings(record=True) as recorded_warnings:
            raise_warning()
            self.assertEqual(
                "Deprecation Message!", six.text_type(recorded_warnings[0].message)
            )

        # raise_warning should show warning until version info is >= (0, 17)
        with warnings.catch_warnings(record=True) as recorded_warnings:
            raise_named_version_warning()
            self.assertEqual(
                "Deprecation Message!", six.text_type(recorded_warnings[0].message)
            )

        # the deprecation warning is not issued because we passed
        # _dont_call_warning
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.warn_until(
                (0, 17), "Foo", _dont_call_warnings=True, _version_info_=(0, 16)
            )
            self.assertEqual(0, len(recorded_warnings))

        # Let's set version info to (0, 17), a RuntimeError should be raised
        with self.assertRaisesRegex(
            RuntimeError,
            r"The warning triggered on filename \'(.*)test_versions.py\', "
            r"line number ([\d]+), is supposed to be shown until version "
            r"0.17.0 is released. Current version is now 0.17.0. "
            r"Please remove the warning.",
        ):
            raise_warning(_version_info_=(0, 17, 0))

        # Let's set version info to (0, 17), a RuntimeError should be raised
        with self.assertRaisesRegex(
            RuntimeError,
            r"The warning triggered on filename \'(.*)test_versions.py\', "
            r"line number ([\d]+), is supposed to be shown until version "
            r"(.*) is released. Current version is now "
            r"([\d.]+). Please remove the warning.",
        ):
            raise_named_version_warning(
                _version_info_=(
                    getattr(sys, "maxint", None) or getattr(sys, "maxsize"),
                    16,
                    0,
                )
            )

        # Even though we're calling warn_until, we pass _dont_call_warnings
        # because we're only after the RuntimeError
        with self.assertRaisesRegex(
            RuntimeError,
            r"The warning triggered on filename \'(.*)test_versions.py\', "
            r"line number ([\d]+), is supposed to be shown until version "
            r"0.17.0 is released. Current version is now "
            r"(.*). Please remove the warning.",
        ):
            salt.utils.versions.warn_until((0, 17), "Foo", _dont_call_warnings=True)

        with self.assertRaisesRegex(
            RuntimeError,
            r"The warning triggered on filename \'(.*)test_versions.py\', "
            r"line number ([\d]+), is supposed to be shown until version "
            r"(.*) is released. Current version is now "
            r"(.*). Please remove the warning.",
        ):
            salt.utils.versions.warn_until(
                "Hydrogen",
                "Foo",
                _dont_call_warnings=True,
                _version_info_=(
                    getattr(sys, "maxint", None) or getattr(sys, "maxsize"),
                    16,
                    0,
                ),
            )

        # version on the deprecation message gets properly formatted
        with warnings.catch_warnings(record=True) as recorded_warnings:
            vrs = salt.version.SaltStackVersion.from_name("Helium")
            salt.utils.versions.warn_until(
                "Helium",
                "Deprecation Message until {version}!",
                _version_info_=(vrs.major - 1, 0),
            )
            self.assertEqual(
                "Deprecation Message until {0}!".format(vrs.formatted_version),
                six.text_type(recorded_warnings[0].message),
            )

    def test_kwargs_warn_until_warning_raised(self):
        # We *always* want *all* warnings thrown on this module
        warnings.filterwarnings("always", "", DeprecationWarning, __name__)

        def raise_warning(**kwargs):
            _version_info_ = kwargs.pop("_version_info_", (0, 16, 0))
            salt.utils.versions.kwargs_warn_until(
                kwargs, (0, 17), _version_info_=_version_info_
            )

        # raise_warning({...}) should show warning until version info is >= (0, 17)
        with warnings.catch_warnings(record=True) as recorded_warnings:
            raise_warning(foo=42)  # with a kwarg
            self.assertEqual(
                "The following parameter(s) have been deprecated and "
                "will be removed in '0.17.0': 'foo'.",
                six.text_type(recorded_warnings[0].message),
            )
        # With no **kwargs, should not show warning until version info is >= (0, 17)
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.kwargs_warn_until(
                {}, (0, 17), _version_info_=(0, 16, 0)  # no kwargs
            )
            self.assertEqual(0, len(recorded_warnings))

        # Let's set version info to (0, 17), a RuntimeError should be raised
        # regardless of whether or not we pass any **kwargs.
        with self.assertRaisesRegex(
            RuntimeError,
            r"The warning triggered on filename \'(.*)test_versions.py\', "
            r"line number ([\d]+), is supposed to be shown until version "
            r"0.17.0 is released. Current version is now 0.17.0. "
            r"Please remove the warning.",
        ):
            raise_warning(_version_info_=(0, 17))  # no kwargs

        with self.assertRaisesRegex(
            RuntimeError,
            r"The warning triggered on filename \'(.*)test_versions.py\', "
            r"line number ([\d]+), is supposed to be shown until version "
            r"0.17.0 is released. Current version is now 0.17.0. "
            r"Please remove the warning.",
        ):
            raise_warning(bar="baz", qux="quux", _version_info_=(0, 17))  # some kwargs

    def test_warn_until_date_warning_raised(self):
        # We *always* want *all* warnings thrown on this module
        warnings.filterwarnings("always", "", DeprecationWarning, __name__)

        _current_date = datetime.date(2000, 1, 1)

        # Test warning with datetime.date instance
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.warn_until_date(
                datetime.date(2000, 1, 2),
                "Deprecation Message!",
                _current_date=_current_date,
            )
            self.assertEqual(
                "Deprecation Message!", six.text_type(recorded_warnings[0].message)
            )

        # Test warning with datetime.datetime instance
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.warn_until_date(
                datetime.datetime(2000, 1, 2),
                "Deprecation Message!",
                _current_date=_current_date,
            )
            self.assertEqual(
                "Deprecation Message!", six.text_type(recorded_warnings[0].message)
            )

        # Test warning with date as a string
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.warn_until_date(
                "20000102", "Deprecation Message!", _current_date=_current_date
            )
            self.assertEqual(
                "Deprecation Message!", six.text_type(recorded_warnings[0].message)
            )

        # the deprecation warning is not issued because we passed
        # _dont_call_warning
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.warn_until_date(
                "20000102",
                "Deprecation Message!",
                _dont_call_warnings=True,
                _current_date=_current_date,
            )
            self.assertEqual(0, len(recorded_warnings))

        # Let's test for RuntimeError raise
        with self.assertRaisesRegex(
            RuntimeError,
            r"Deprecation Message! This warning\(now exception\) triggered on "
            r"filename \'(.*)test_versions.py\', line number ([\d]+), is "
            r"supposed to be shown until ([\d-]+). Today is ([\d-]+). "
            r"Please remove the warning.",
        ):
            salt.utils.versions.warn_until_date("20000101", "Deprecation Message!")

        # Even though we're calling warn_until_date, we pass _dont_call_warnings
        # because we're only after the RuntimeError
        with self.assertRaisesRegex(
            RuntimeError,
            r"Deprecation Message! This warning\(now exception\) triggered on "
            r"filename \'(.*)test_versions.py\', line number ([\d]+), is "
            r"supposed to be shown until ([\d-]+). Today is ([\d-]+). "
            r"Please remove the warning.",
        ):
            salt.utils.versions.warn_until_date(
                "20000101",
                "Deprecation Message!",
                _dont_call_warnings=True,
                _current_date=_current_date,
            )

    def test_warn_until_date_bad_strptime_format(self):
        # We *always* want *all* warnings thrown on this module
        warnings.filterwarnings("always", "", DeprecationWarning, __name__)

        # Let's test for RuntimeError raise
        with self.assertRaisesRegex(
            ValueError, "time data '0022' does not match format '%Y%m%d'"
        ):
            salt.utils.versions.warn_until_date("0022", "Deprecation Message!")



# -*- coding: utf-8 -*-
# pylint: disable=resource-leakage

# Import Python libs
from __future__ import absolute_import, print_function

import hashlib
import optparse
import pprint
import subprocess

# Import Salt libs
import salt.utils.color
import salt.utils.files
import salt.utils.yaml

# Import 3rd-party libs
from salt.ext import six

colors = salt.utils.color.get_colors()


def parse():
    """
    Parse command line options
    """
    parser = optparse.OptionParser()
    parser.add_option(
        "-r",
        "--runs",
        dest="runs",
        default=10,
        type=int,
        help="Specify the number of times to run the consistency check",
    )
    parser.add_option(
        "-c",
        "--command",
        dest="command",
        default="state.show_highstate",
        help="The command to execute",
    )

    options, args = parser.parse_args()
    return options.__dict__


def run(command):
    """
    Execute a single command and check the returns
    """
    cmd = r"salt \* {0} --yaml-out -t 500 > high".format(command)
    subprocess.call(cmd, shell=True)
    with salt.utils.files.fopen("high") as fp_:
        data = salt.utils.yaml.safe_load(fp_)
    hashes = set()
    for key, val in six.iteritems(data):
        has = hashlib.md5(str(val)).hexdigest()
        if has not in hashes:
            print("{0}:".format(has))
            pprint.pprint(val)
        hashes.add(has)
    if len(hashes) > 1:
        print(
            "{0}Command: {1} gave inconsistent returns{2}".format(
                colors["LIGHT_RED"], command, colors["ENDC"]
            )
        )


if __name__ == "__main__":
    opts = parse()
    for _ in opts["runs"]:
        for command in opts["command"].split(","):
            print("-" * 30)
            print("Running command {0}".format(command))
            run(command)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.sysrc as sysrc

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SysrcTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the sysrc state
    """

    def setup_loader_modules(self):
        return {sysrc: {}}

    def test_managed(self):
        """
            Test to ensure a sysrc variable is set to a specific value.
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        mock = MagicMock(side_effect=[{"key1": {"salt": "stack"}}, None, None])
        mock1 = MagicMock(return_value=True)
        with patch.dict(sysrc.__salt__, {"sysrc.get": mock, "sysrc.set": mock1}):
            ret.update({"comment": "salt is already set to the desired" " value."})
            self.assertDictEqual(sysrc.managed("salt", "stack"), ret)

            with patch.dict(sysrc.__opts__, {"test": True}):
                ret.update(
                    {
                        "changes": {"new": "salt = stack will be set.", "old": None},
                        "comment": "The value" ' of "salt" will be changed!',
                        "result": None,
                    }
                )
                self.assertDictEqual(sysrc.managed("salt", "stack"), ret)

            with patch.dict(sysrc.__opts__, {"test": False}):
                ret.update(
                    {
                        "changes": {"new": True, "old": None},
                        "comment": 'The value of "salt" was changed!',
                        "result": True,
                    }
                )
                self.assertDictEqual(sysrc.managed("salt", "stack"), ret)

    def test_absent(self):
        """
            Test to ensure a sysrc variable is absent.
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        mock = MagicMock(side_effect=[None, True, True])
        mock1 = MagicMock(return_value=True)
        with patch.dict(sysrc.__salt__, {"sysrc.get": mock, "sysrc.remove": mock1}):
            ret.update({"comment": '"salt" is already absent.'})
            self.assertDictEqual(sysrc.absent("salt"), ret)

            with patch.dict(sysrc.__opts__, {"test": True}):
                ret.update(
                    {
                        "changes": {"new": '"salt" will be removed.', "old": True},
                        "comment": '"salt" will be removed!',
                        "result": None,
                    }
                )
                self.assertDictEqual(sysrc.absent("salt"), ret)

            with patch.dict(sysrc.__opts__, {"test": False}):
                ret.update(
                    {
                        "changes": {"new": True, "old": True},
                        "comment": '"salt" was removed!',
                        "result": True,
                    }
                )
                self.assertDictEqual(sysrc.absent("salt"), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
    :copyright: Copyright 2017 by the SaltStack Team, see AUTHORS for more details.


    tests.unit.beacons.test_status
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Status beacon test cases
"""
from __future__ import absolute_import

import salt.config
import salt.loader
import salt.modules.status as status_module
from salt.beacons import status
from tests.support.helpers import slowTest
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase


class StatusBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.status
    """

    def setup_loader_modules(self):
        opts = salt.config.DEFAULT_MINION_OPTS.copy()
        opts["grains"] = salt.loader.grains(opts)
        module_globals = {
            "__opts__": opts,
            "__salt__": "autoload",
            "__context__": {},
            "__grains__": opts["grains"],
        }
        return {status: module_globals, status_module: module_globals}

    @slowTest
    def test_empty_config(self, *args, **kwargs):
        config = []

        ret = status.validate(config)
        self.assertEqual(ret, (True, "Valid beacon configuration"))

        ret = status.beacon(config)
        expected = sorted(["loadavg", "meminfo", "cpustats", "vmstats", "time"])

        self.assertEqual(sorted(list(ret[0]["data"])), expected)

    @slowTest
    def test_deprecated_dict_config(self):
        config = {"time": ["all"]}

        ret = status.validate(config)
        self.assertEqual(
            ret, (False, "Configuration for status beacon must be a list.")
        )

    @slowTest
    def test_list_config(self):
        config = [{"time": ["all"]}]

        ret = status.validate(config)
        self.assertEqual(ret, (True, "Valid beacon configuration"))

        ret = status.beacon(config)
        expected = ["time"]

        self.assertEqual(list(ret[0]["data"]), expected)
